# Generated by Django 4.2.2 on 2024-05-08 08:18

from django.db import migrations, models
import django.db.models.deletion
import django_fsm


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('onestopdocuments', '0003_auto_20221002_1251'),
    ]

    operations = [
        migrations.CreateModel(
            name='OpenSignDocumentTracker',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('opensign_document_id', models.CharField(max_length=255)),
                ('status', django_fsm.FSMField(choices=[('PENDING', 'Pending'), ('SIGNED', 'Signed'), ('CANCELLED', 'Cancelled')], default='PENDING', max_length=20)),
                ('generated_document', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='onestopdocuments.generateddocument')),
            ],
            options={
                'db_table': 'opensign_document_tracker',
                'unique_together': {('generated_document', 'opensign_document_id', 'status')},
            },
        ),
        migrations.CreateModel(
            name='OpenSignSignerTracker',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('signer_email', models.EmailField(max_length=254)),
                ('signer_url', models.URLField()),
                ('status', django_fsm.FSMField(choices=[('PENDING', 'Pending'), ('SIGNED', 'Signed'), ('CANCELLED', 'Cancelled')], default='PENDING', max_length=20)),
                ('document_tracker', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='signer_trackers', to='onestop_opensign.opensigndocumenttracker')),
            ],
            options={
                'db_table': 'opensign_signer_tracker',
                'unique_together': {('document_tracker', 'signer_email')},
            },
        ),
    ]
